Package: pls
Title: Partial Least Squares and Principal Component Regression
Version: 2.9-0
Date: 2026-02-20
Authors@R: c(person("Kristian Hovde", "Liland", role = c("aut", "cre"),
                    email = "kristian.liland@nmbu.no"),
	     person("Bjørn-Helge", "Mevik", role = "aut" ,
                    email = "b-h@mevik.net"),
             person("Ron", "Wehrens", role = "aut"),
             person("Paul",  "Hiemstra", role = "ctb"))
Author: Kristian Hovde Liland [aut, cre],
  Bjørn-Helge Mevik [aut],
  Ron Wehrens [aut],
  Paul Hiemstra [ctb]
Maintainer: Kristian Hovde Liland <kristian.liland@nmbu.no>
Encoding: UTF-8
LazyData: yes
Description: Multivariate regression methods
	Partial Least Squares Regression (PLSR), Principal Component
	Regression (PCR) and Canonical Powered Partial Least Squares (CPPLS).
Depends: R (>= 3.5.0)
Imports: grDevices, graphics, methods, stats
Suggests: MASS, parallel, Rmpi, testthat, RUnit, knitr
License: GPL-2
URL: https://github.com/khliland/pls, https://khliland.github.io/pls/
BugReports: https://github.com/khliland/pls/issues
Config/testthat/edition: 2
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-02-20 20:26:00 UTC; kristian
Repository: CRAN
Date/Publication: 2026-02-21 06:10:07 UTC
Built: R 4.6.0; ; 2026-04-25 16:27:32 UTC; unix
