# Generated by roxygen2: do not edit by hand

S3method(MSEP,mvr)
S3method(R2,mvr)
S3method(RMSEP,mvr)
S3method(as.data.frame,mvrVal)
S3method(biplot,mvr)
S3method(coef,mvr)
S3method(fitted,mvr)
S3method(loadingplot,default)
S3method(loadings,default)
S3method(makepredictcall,msc)
S3method(makepredictcall,stdized)
S3method(model.frame,mvr)
S3method(model.matrix,mvr)
S3method(plot,loadings)
S3method(plot,mvr)
S3method(plot,mvrVal)
S3method(plot,scores)
S3method(predict,msc)
S3method(predict,mvr)
S3method(predict,stdized)
S3method(predplot,default)
S3method(predplot,mvr)
S3method(print,jacktest)
S3method(print,mvr)
S3method(print,mvrVal)
S3method(residuals,mvr)
S3method(scoreplot,default)
S3method(scores,default)
S3method(summary,mvr)
S3method(vcov,mvr)
export(MSEP)
export(R2)
export(RMSEP)
export(Yloadings)
export(Yscores)
export(coefplot)
export(compnames)
export(corrplot)
export(cppls)
export(cppls.fit)
export(crossval)
export(cvsegments)
export(explvar)
export(fac2seg)
export(jack.test)
export(kernelpls.fit)
export(loading.weights)
export(loadingplot)
export(loadings)
export(msc)
export(mvr)
export(mvrValstats)
export(nipals)
export(nipals.fit)
export(nipalspc.fit)
export(nipalspcr)
export(oscorespls.fit)
export(pcr)
export(pls.options)
export(plsr)
export(prednames)
export(predplot)
export(predplotXy)
export(respnames)
export(scoreplot)
export(scores)
export(selectNcomp)
export(simpls.fit)
export(stdize)
export(svdpc.fit)
export(validationplot)
export(var.jack)
export(widekernelpls.fit)
importFrom(grDevices,dev.interactive)
importFrom(grDevices,devAskNewPage)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,axTicks)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,matplot)
importFrom(graphics,mtext)
importFrom(graphics,pairs)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(methods,hasArg)
importFrom(stats,.checkMFClasses)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,delete.response)
importFrom(stats,drop.terms)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,makepredictcall)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,napredict)
importFrom(stats,optimize)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,pt)
importFrom(stats,residuals)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stats,var)
