#' @title Partial Least Squares and Principal Component Regression
#'
#' @description
#' Multivariate regression methods Partial Least Squares Regression (PLSR), Principal Component
#' Regression (PCR) and Canonical Powered Partial Least Squares (CPPLS).
#'
#' @importFrom grDevices dev.interactive devAskNewPage
#' @importFrom graphics abline arrows axTicks axis legend matplot mtext pairs par plot points segments text title
#' @importFrom methods hasArg
#' @importFrom stats .checkMFClasses coef cor delete.response drop.terms fitted formula makepredictcall median model.frame model.matrix model.response na.omit na.pass napredict optimize predict printCoefmat pt residuals runif sd terms update var
#' @docType package
#' @name pls
#' @keywords internal
"_PACKAGE"
